#ifndef _LM_H
#define _LM_H
#if __GNUC__ >=3
#pragma GCC system_header
#endif

#include <lmcons.h>
#include <lmaccess.h>
#include <lmalert.h>
#include <lmat.h>
#include <lmaudit.h>
#include <lmchdev.h>
#include <lmconfig.h>
#include <lmerr.h>
#include <lmmsg.h>
#include <lmshare.h>
#include <lmapibuf.h>
#include <lmremutl.h>
#include <lmrepl.h>
#include <lmuse.h>
#include <lmerrlog.h>
#include <lmsvc.h>
#include <lmwksta.h>
#include <lmserver.h>
#include <lmstats.h>

#endif
